import pytest

import rustoml

cases = [
    (
        """\
something = true
lion = "aslan"
""",
        {"something": True, "lion": "aslan"},
    ),
    (
        """\
[section]
z = "last"
a = "first"

[default]
dir = "/home"
beta = true
""",
        {"section": {"z": "last", "a": "first"}, "default": {"dir": "/home", "beta": True}},
    ),
]


@pytest.mark.parametrize("toml_string,python_object", cases)
def test_load_order(toml_string, python_object):
    loaded = rustoml.load(toml_string)
    assert loaded == python_object
    assert list(loaded.items()) == list(python_object.items())  # check order is maintained


@pytest.mark.parametrize("toml_string,python_object", cases)
def test_dump_order(toml_string, python_object):
    assert rustoml.dumps(python_object) == toml_string



from datetime import date, datetime, time, timezone

import pytest

import rustoml


@pytest.mark.parametrize(
    "input_obj,output_toml",
    [
        ({"text": "\nfoo\nbar\n"}, 'text = "\\nfoo\\nbar\\n"\n'),
        ({"foo": "bar"}, 'foo = "bar"\n'),
        (None, '"!NONE"'),
        ([1, 2, 3], "[1, 2, 3]"),
        ([1, 2, None], '[1, 2, "!NONE"]'),
        (datetime(1979, 5, 27, 7, 32), "1979-05-27T07:32:00"),
        (datetime(1979, 5, 27, 7, 32, tzinfo=timezone.utc), "1979-05-27T07:32:00Z"),
        (date(2022, 12, 31), "2022-12-31"),
        (time(12, 00, 59, 23456), "12:00:59.023456"),
        ({"x": datetime(1979, 5, 27, 7, 32)}, "x = 1979-05-27T07:32:00\n"),
        # order changed to avoid https://github.com/alexcrichton/toml-rs/issues/142
        ({"x": {"a": 1}, "y": 4}, "y = 4\n\n[x]\na = 1\n"),
        ({"x": 1, "y": None}, 'x = 1\ny = "!NONE"\n'),
        ((1, 2, 3), "[1, 2, 3]"),
        ((1, 2, None), '[1, 2, "!NONE"]'),
        ({"emoji": "😷"}, 'emoji = "😷"\n'),
        ({"bytes": b"123"}, "bytes = [49, 50, 51]\n"),  # TODO: should this be a string of "123"
        ({"polish": "Witaj świecie"}, 'polish = "Witaj świecie"\n'),
    ],
)
def test_dumps(input_obj, output_toml):
    assert rustoml.dumps(input_obj, none_value="!NONE") == output_toml

    @pytest.mark.parametrize(
        "input_obj,output_toml",
        [
            ([1, 2, None], "[1, 2]"),
            ((1, 2, None), "[1, 2]"),
            (None, "null"),
        ],
    )
    def test_dumps_no_none(input_obj, output_toml):
        assert rustoml.dumps(input_obj, none_value=None) == output_toml


@pytest.mark.parametrize(
    "input_obj,output_toml",
    [
        ({"text": "\nfoo\nbar\n"}, "text = '''\n\nfoo\nbar\n'''\n"),
        ({"foo": "bar"}, "foo = 'bar'\n"),
        ([1, 2, 3], "[\n    1,\n    2,\n    3,\n]"),
        ((1, 2, 3), "[\n    1,\n    2,\n    3,\n]"),
    ],
)
def test_dumps_pretty(input_obj, output_toml):
    assert rustoml.dumps(input_obj, pretty=True) == output_toml


@pytest.mark.parametrize(
    "input_obj,output_toml,size",
    [
        ({"foo": "bar"}, 'foo = "bar"\n', 12),
        ({"emoji": "😷"}, 'emoji = "😷"\n', 12),
        ({"polish": "Witaj świecie"}, 'polish = "Witaj świecie"\n', 25),
    ],
)
def test_dump_path(tmp_path, input_obj, output_toml, size):
    p = tmp_path / "test.toml"
    assert rustoml.dump(input_obj, p) == size
    assert p.read_text(encoding="UTF-8") == output_toml


def test_dump_file(tmp_path):
    p = tmp_path / "test.toml"
    with p.open("w") as f:
        assert rustoml.dump({"foo": "bar"}, f) == 12
    assert p.read_text() == 'foo = "bar"\n'


def test_varied_list():
    assert rustoml.dumps({"test": [1, "2"]}) == 'test = [1, "2"]\n'


@pytest.mark.parametrize(
    "input_obj, none_value, output_toml",
    [
        ({"test": None}, "null", 'test = "null"\n'),
        ({"test": None}, "foo", 'test = "foo"\n'),
        ({"test": None}, None, ""),
        ({None: "test"}, "null", 'null = "test"\n'),
        ({None: "test"}, "foo", 'foo = "test"\n'),
        ({None: "test"}, None, ""),
        ({"test": [1, None, 2]}, "null", 'test = [1, "null", 2]\n'),
        ({"test": [1, None, 2]}, "foo", 'test = [1, "foo", 2]\n'),
        ({"test": [1, None, 2]}, None, "test = [1, 2]\n"),
        (
            {"test": {"x": [{"y": [1, None, 2]}], "z": None}},
            "null",
            '[test]\nz = "null"\n\n[[test.x]]\ny = [1, "null", 2]\n',
        ),
        (
            {"test": {"x": [{"y": [1, None, 2]}], "z": None}},
            "foo",
            '[test]\nz = "foo"\n\n[[test.x]]\ny = [1, "foo", 2]\n',
        ),
        ({"test": {"x": [{"y": [1, None, 2]}], "z": None}}, None, "[[test.x]]\ny = [1, 2]\n"),
    ],
)
def test_none_value(input_obj, none_value, output_toml):
    assert rustoml.dumps(input_obj, none_value=none_value) == output_toml



from importlib.machinery import SourceFileLoader
from pathlib import Path
from types import ModuleType

import pytest

import rustoml


def test_example():
    loader = SourceFileLoader("example", str(Path(__file__).parent / "../example.py"))
    module = ModuleType(loader.name)
    loader.exec_module(module)
    # check it looks about right
    assert isinstance(module.obj, dict)
    assert module.obj["title"] == "TOML Example"


def test_version():
    assert isinstance(rustoml.__version__, str)
    print("rustoml __version__:", rustoml.__version__)


@pytest.mark.parametrize(
    "dt",
    [
        # passes, for 6 significant subsecond digits
        "2020-05-25T12:00:01.123456",
        # previously failed because last subsecond digit is not dumped to string
        # and load parsing fails with only 5 fractional digits
        "2020-05-25T12:00:01.123450",
    ],
)
def test_datetime_precision(dt):
    assert rustoml.loads(rustoml.dumps({"a": dt})) == {"a": dt}



"""
rustoml - A high-performance TOML library for Python.

rustoml is a Python library for parsing and serializing TOML (Tom's Obvious, Minimal Language)
implemented in Rust for maximum performance. It provides a simple API similar to the standard
library's json module.

Key features:
- Fast: Built on rust's toml crate for high performance
- Correct: Passes all standard TOML tests
- Flexible: Supports custom None value handling
- Typed: Full type annotations and py.typed marker

Example:
    >>> import rustoml
    >>> data = rustoml.loads('[package]\\nname = "rustoml"')
    >>> print(data)
    {'package': {'name': 'rustoml'}}
    >>> rustoml.dumps(data)
    '[package]\\nname = "rustoml"\\n'
"""

import re
from collections.abc import Callable
from io import TextIOBase
from pathlib import Path
from typing import Any, BinaryIO, TextIO

from . import _rustoml

__all__ = (
    "__version__",
    "VERSION",
    "TomlParsingError",
    "TomlSerializationError",
    "load",
    "loads",
    "dumps",
    "dump",
)

# VERSION is set in Cargo.toml
VERSION: str = _rustoml.__version__
"""The version of the rustoml package (alias for __version__)."""

__version__: str = _rustoml.__version__
"""The version of the rustoml package."""


class TomlParsingError(_rustoml.TomlParsingError):
    """
    Exception raised when TOML parsing fails.

    This exception provides detailed information about parsing failures,
    similar to Python's tomllib.TOMLDecodeError.

    Attributes:
        msg: The error message describing what went wrong
        doc: The TOML document that failed to parse (if available)
        pos: The character position where parsing failed (0-indexed)
        lineno: The line number where parsing failed (1-indexed)
        colno: The column number where parsing failed (1-indexed)
    """

    def __init__(self, message: str, doc: str | None = None):
        super().__init__(message)
        self.msg = message
        self.doc = doc
        self.pos: int | None = None
        self.lineno: int | None = None
        self.colno: int | None = None

        # Parse error message to extract line and column info
        # Format: "TOML parse error at line X, column Y"
        match = re.search(r"at line (\d+), column (\d+)", message)
        if match:
            self.lineno = int(match.group(1))
            self.colno = int(match.group(2))

            # Calculate position if we have the document
            if doc is not None:
                self.pos = self._calculate_position(doc, self.lineno, self.colno)

    def _calculate_position(self, doc: str, lineno: int, colno: int) -> int:
        """Calculate the character position from line and column numbers."""
        lines = doc.splitlines(keepends=True)
        pos = 0
        for i, line in enumerate(lines, 1):
            if i == lineno:
                return pos + colno - 1
            pos += len(line)
        return pos

    def __str__(self) -> str:
        """Return a formatted error message with context."""
        if self.lineno is not None and self.colno is not None and self.doc is not None:
            lines = self.doc.splitlines()
            error_lines = [f"TOML parse error at line {self.lineno}, column {self.colno}"]

            # Show the problematic line with context
            if 0 < self.lineno <= len(lines):
                # Show previous line if available
                if self.lineno > 1:
                    error_lines.append(f"{self.lineno - 1:4d} | {lines[self.lineno - 2]}")

                # Show the error line
                error_line = lines[self.lineno - 1]
                error_lines.append(f"{self.lineno:4d} | {error_line}")

                # Show the error pointer
                pointer = " " * (7 + self.colno - 1) + "^"
                error_lines.append(pointer)

                # Show next line if available
                if self.lineno < len(lines):
                    error_lines.append(f"{self.lineno + 1:4d} | {lines[self.lineno]}")

            # Extract the actual error message
            msg_match = re.search(r"\n(.+)$", self.msg)
            if msg_match:
                error_lines.append("")
                error_lines.append(msg_match.group(1))

            return "\n".join(error_lines)
        return self.msg


class TomlSerializationError(_rustoml.TomlSerializationError):
    """Exception raised when TOML serialization fails."""

    pass


def _apply_parse_float(obj: Any, parse_float: Callable[[str], Any]) -> Any:
    """Recursively apply parse_float to all float values in a data structure."""
    if isinstance(obj, float):
        # Convert float back to string and parse with custom function
        return parse_float(str(obj))
    elif isinstance(obj, dict):
        return {k: _apply_parse_float(v, parse_float) for k, v in obj.items()}
    elif isinstance(obj, list):
        return [_apply_parse_float(item, parse_float) for item in obj]
    return obj


def load(
    toml: str | Path | TextIO | BinaryIO,
    *,
    none_value: str | None = None,
    parse_float: Callable[[str], Any] = float,
) -> dict[str, Any]:
    """
    Parse TOML from a string, file path, or file object and return a Python dictionary.

    This function provides a convenient interface for loading TOML data from various sources.
    It automatically handles reading from files and delegates to `loads()` for parsing.

    Args:
        toml: The TOML source to parse. Can be:
            - A `str` containing TOML data
            - A `Path` object pointing to a TOML file
            - A file object opened in text or binary mode
        none_value: Optional string value to treat as Python `None`. When specified,
            any string matching this value in the TOML will be converted to `None`.
            By default, no values are converted to `None`.
        parse_float: A callable that will be called with the string representation
            of every TOML float to convert it to a Python object. Defaults to `float`.
            Can be used with `decimal.Decimal` for exact precision.

    Returns:
        A dictionary representing the parsed TOML data with string keys and
        values of various types (str, int, float, bool, list, dict, datetime).

    Raises:
        TomlParsingError: If the TOML data is malformed or invalid.
        TypeError: If `toml` is not a str, Path, or file object.
        OSError: If reading from a Path or file object fails.

    Example:
        >>> import rustoml
        >>> from pathlib import Path
        >>> from decimal import Decimal
        >>> # From string
        >>> data = rustoml.load('[package]\\nname = "example"')
        >>> # From Path
        >>> data = rustoml.load(Path('config.toml'))
        >>> # From binary file object
        >>> with open('config.toml', 'rb') as f:
        ...     data = rustoml.load(f)
        >>> # With custom float parsing
        >>> data = rustoml.load('[price]\\nvalue = 19.99', parse_float=Decimal)
    """
    if isinstance(toml, Path):
        toml_str = toml.read_text(encoding="UTF-8")
    elif isinstance(toml, str):
        toml_str = toml
    elif isinstance(toml, (TextIOBase, TextIO)):
        toml_str = toml.read()
    elif isinstance(toml, BinaryIO):
        # Binary file mode - decode as UTF-8
        content = toml.read()
        toml_str = content.decode("utf-8")
    else:
        # Try to handle file-like objects with a mode attribute
        if hasattr(toml, "read") and callable(getattr(toml, "read")):
            mode_attr = getattr(toml, "mode", None)
            if isinstance(mode_attr, str) and "b" in mode_attr:
                # Binary mode
                content = toml.read()  # type: ignore[attr-defined]
                toml_str = content.decode("utf-8") if isinstance(content, bytes) else str(content)
            else:
                # Text mode
                toml_str = str(toml.read())  # type: ignore[attr-defined]
        else:
            raise TypeError(f"invalid toml input type: {type(toml)}")

    return loads(toml_str, none_value=none_value, parse_float=parse_float)


def loads(
    toml: str,
    *,
    none_value: str | None = None,
    parse_float: Callable[[str], Any] = float,
) -> dict[str, Any]:
    """
    Parse a TOML string and return a Python dictionary.

    This function parses TOML data from a string. The interface matches that of
    `json.loads()` for consistency with Python's standard library.

    Args:
        toml: A string containing valid TOML data.
        none_value: Optional string value to treat as Python `None`. When specified,
            any string matching this value in the TOML will be converted to `None`.
            By default, no values are converted to `None`.
        parse_float: A callable that will be called with the string representation
            of every TOML float to convert it to a Python object. Defaults to `float`.
            Can be used with `decimal.Decimal` for exact precision.

    Returns:
        A dictionary representing the parsed TOML data with string keys and
        values of various types (str, int, float/custom, bool, list, dict, datetime).

    Raises:
        TomlParsingError: If the TOML string is malformed or invalid.
        TypeError: If `toml` is not a string.

    Example:
        >>> import rustoml
        >>> from decimal import Decimal
        >>> toml_str = '''
        ... [package]
        ... name = "rustoml"
        ... version = "0.12.0"
        ... '''
        >>> data = rustoml.loads(toml_str)
        >>> print(data['package']['name'])
        rustoml
        >>> # With Decimal for exact precision
        >>> data = rustoml.loads('price = 19.99', parse_float=Decimal)
        >>> isinstance(data['price'], Decimal)
        True
    """
    if not isinstance(toml, str):
        raise TypeError(f"invalid toml input, must be str not {type(toml)}")
    try:
        result = _rustoml.deserialize(toml, none_value=none_value)
        # Apply custom float parsing if not using default
        if parse_float is not float:
            result = _apply_parse_float(result, parse_float)
        return result
    except _rustoml.TomlParsingError as e:
        # Wrap the Rust error with our enhanced Python error
        raise TomlParsingError(str(e), doc=toml) from None


def dumps(obj: Any, *, pretty: bool = True, none_value: str | None = "null") -> str:
    """
    Serialize a Python object to a TOML string.

    This function converts a Python object (typically a dictionary) to TOML format.
    The interface matches that of `json.dumps()` for consistency with Python's standard library.

    Args:
        obj: A Python object to serialize. Typically a dict, but can also contain
            lists, strings, numbers, booleans, datetime objects, and None values.
        pretty: If `True`, the output uses a more readable "pretty" format with
            better spacing and organization. Default is `True`.
        none_value: Controls how `None` values are serialized. If a string is provided,
            `None` values will be serialized as that string. If `None` is passed,
            `None` values will be omitted from the output. Default is `'null'`.

    Returns:
        A TOML string representation of the object.

    Raises:
        TomlSerializationError: If the object cannot be serialized to TOML
            (e.g., unsupported types, circular references).

    Example:
        >>> import rustoml
        >>> data = {'package': {'name': 'rustoml', 'version': '0.13.0'}}
        >>> print(rustoml.dumps(data))
        [package]
        name = "rustoml"
        version = "0.13.0"
        <BLANKLINE>
        >>> # With None handling
        >>> data_with_none = {'key': None, 'other': 'value'}
        >>> rustoml.dumps(data_with_none, none_value=None)  # Omit None
        'other = "value"\\n'
        >>> rustoml.dumps(data_with_none, none_value='@null')  # Serialize as '@null'
        'key = "@null"\\nother = "value"\\n'
    """
    if pretty:
        serialize = _rustoml.serialize_pretty
    else:
        serialize = _rustoml.serialize

    return serialize(obj, none_value=none_value)


def dump(obj: Any, file: Path | TextIO, *, pretty: bool = False, none_value: str | None = "null") -> int:
    """
    Serialize a Python object to TOML and write it to a file.

    This function converts a Python object to TOML format and writes it directly to a file.
    The interface matches that of `json.dump()` for consistency with Python's standard library.

    Args:
        obj: A Python object to serialize. Typically a dict, but can also contain
            lists, strings, numbers, booleans, datetime objects, and None values.
        file: The destination for the TOML output. Can be:
            - A `Path` object pointing to the destination file
            - A file object opened in text write mode
        pretty: If `True`, the output uses a more readable "pretty" format with
            better spacing and organization. Default is `False`.
        none_value: Controls how `None` values are serialized. If a string is provided,
            `None` values will be serialized as that string. If `None` is passed,
            `None` values will be omitted from the output. Default is `'null'`.

    Returns:
        The number of characters written to the file.

    Raises:
        TomlSerializationError: If the object cannot be serialized to TOML.
        OSError: If writing to the file fails.
        TypeError: If `file` is not a Path or file object.

    Example:
        >>> import rustoml
        >>> from pathlib import Path
        >>> data = {'package': {'name': 'rustoml', 'version': '0.13.0'}}
        >>> # Write to Path
        >>> rustoml.dump(data, Path('config.toml'), pretty=True)
        >>> # Write to file object
        >>> with open('config.toml', 'w') as f:
        ...     rustoml.dump(data, f, pretty=True)
    """
    s = dumps(obj, pretty=pretty, none_value=none_value)
    if isinstance(file, Path):
        return file.write_text(s, encoding="UTF-8")
    else:
        return file.write(s)



import sys
from datetime import date, datetime, time, timedelta, timezone
from pathlib import Path

import pytest
from dirty_equals import HasRepr

import rustoml

data_toml_path = Path("tests/data.toml")

pytestmark = pytest.mark.skipif(sys.version_info < (3, 8), reason="requires python3.8 or higher")


def test_data():
    data = rustoml.load(data_toml_path)
    assert data == {
        "title": "TOML Example",
        "owner": {
            "name": "Tom Preston-Werner",
            "dob": datetime(1979, 5, 27, 7, 32, tzinfo=timezone(timedelta(hours=-8))),
        },
        "database": {"server": "192.168.1.1", "ports": [8001, 8001, 8002], "connection_max": 5000, "enabled": True},
        "servers": {"alpha": {"ip": "10.0.0.1", "dc": "eqdc10"}, "beta": {"ip": "10.0.0.2", "dc": "eqdc10"}},
        "clients": {
            "data": [["gamma", "delta"], [1, 2]],
            "hosts": ["alpha", "omega"],
            "apple": {"type": "fruit", "skin": "thin", "color": "red"},
            "orange": {"type": "fruit", "skin": "thick", "color": "orange"},
        },
        "strings": {
            "str0": "Roses are red\nViolets are blue",
            "str1": "The quick brown fox jumps over the lazy dog.",
            "str2": "The quick brown fox jumps over the lazy dog.",
            "str3": "The quick brown fox jumps over the lazy dog.",
            "str4": 'Here are two quotation marks: "". Simple enough.',
            "str5": 'Here are three quotation marks: """.',
            "str6": 'Here are fifteen quotation marks: """"""""""""""".',
            "winpath": "C:\\Users\\nodejs\\templates",
            "winpath2": "\\\\ServerX\\admin$\\system32\\",
            "quoted": 'Tom "Dubs" Preston-Werner',
            "regex": "<\\i\\c*\\s*>",
            "regex2": "I [dw]on't need \\d{2} apples",
            "lines": "The first newline is\ntrimmed in raw strings.\n   All other whitespace\n   is preserved.\n",
            "quot15": 'Here are fifteen quotation marks: """""""""""""""',
            "apos15": "Here are fifteen apostrophes: '''''''''''''''",
        },
        "numbers": {
            "int1": 99,
            "int2": 42,
            "int3": 0,
            "int4": -17,
            "int5": 1000,
            "int6": 5349221,
            "int7": 12345,
            "hex1": 3735928559,
            "hex2": 3735928559,
            "hex3": 3735928559,
            "oct1": 342391,
            "oct2": 493,
            "bin1": 214,
            "flt1": 1.0,
            "flt2": 3.1415,
            "flt3": -0.01,
            "flt4": 5e22,
            "flt5": 1000000.0,
            "flt6": -0.02,
            "flt7": 6.626e-34,
            "sf1": HasRepr("inf"),
            "sf2": HasRepr("inf"),
            "sf3": HasRepr("-inf"),
            "sf4": HasRepr("nan"),
            "sf5": HasRepr("nan"),
            "sf6": HasRepr("nan"),
            "bool1": True,
            "bool2": False,
        },
        "dates": {
            "odt1": datetime(1979, 5, 27, 7, 32, tzinfo=timezone.utc),
            "odt2": datetime(1979, 5, 27, 0, 32, tzinfo=timezone(timedelta(hours=-7))),
            "odt3": datetime(1979, 5, 27, 0, 32, 0, 999999, tzinfo=timezone(timedelta(hours=-7))),
            "ldt1": datetime(1979, 5, 27, 7, 32),
            "ldt2": datetime(1979, 5, 27, 0, 32, 0, 999999),
            "ld1": date(1979, 5, 27),
            "lt1": time(7, 32),
            "lt2": time(0, 32, 0, 999999),
        },
        "lists": {
            "integers": [1, 2, 3],
            "colors": ["red", "yellow", "green"],
            "nested_array_of_int": [[1, 2], [3, 4, 5]],
            "nested_mixed_array": [[1, 2], ["a", "b", "c"]],
            "integers2": [1, 2, 3],
            "integers3": [1, 2],
        },
        "tables": {
            "table-1": {"key1": "some string", "key2": 123},
            "table-2": {"key1": "another string", "key2": 456},
            "dog": {"tater.man": {"type": {"name": "pug"}}},
            "x": {"y": {"z": {"w": {}}}},
            "product": {"type": {"name": "Nail"}},
        },
        "fruit": {
            "apple": {"color": "red", "taste": {"sweet": True}},
            "tables": {
                "name": {"first": "Tom", "last": "Preston-Werner"},
                "point": {"x": 1, "y": 2},
                "animal": {"type": {"name": "pug"}},
            },
            "variety": [
                {
                    "name": "plantain",
                    "points": [{"x": 1, "y": 2, "z": 3}, {"x": 7, "y": 8, "z": 9}, {"x": 2, "y": 4, "z": 8}],
                }
            ],
        },
        "products": [{"name": "Hammer", "sku": 738594937}, {}, {"name": "Nail", "sku": 284758393, "color": "gray"}],
        "spoons": [
            {
                "name": "apple",
                "physical": {"color": "red", "shape": "round"},
                "variety": [{"name": "red delicious"}, {"name": "granny smith"}],
            },
            {"name": "banana"},
        ],
    }


@pytest.mark.benchmark(group="load-data.toml")
def test_loads_data_toml(benchmark):
    data_toml_str = data_toml_path.read_text()
    benchmark(rustoml.loads, data_toml_str)


@pytest.mark.benchmark(group="load-data.toml")
def test_load_data_toml(benchmark):
    benchmark(rustoml.load, data_toml_path)


@pytest.mark.benchmark(group="load-dict")
def test_load_big_dict(benchmark):
    d = {str(i): i for i in range(1000)}
    toml_str = rustoml.dumps(d)
    benchmark(rustoml.loads, toml_str)


@pytest.mark.benchmark(group="dumps-data.toml")
def test_dump_data_toml(benchmark):
    data = rustoml.load(data_toml_path)
    # benchmark(rustoml.loads, data_toml_bytes, name='loads-bytes')
    benchmark(rustoml.dumps, data)


@pytest.mark.benchmark(group="dumps-simple")
def test_dump_simple(benchmark):
    data = {
        "fruit": ["apple", "orange", "banana", None, True, False, 123, 123.456, 1e-10, 1e10],
        "tables": {"name": "Tom", "point": {"x": 1, "y": 2}},
        "datetime": datetime(1979, 5, 27, 7, 32, tzinfo=timezone.utc),
        "tuples": [(i + 1, i + 2, i + 3) for i in range(10)],
        "many": [{str(i): i + j for i in range(100)} for j in range(100)],
    }

    # benchmark(rustoml.loads, data_toml_bytes, name='loads-bytes')
    benchmark(rustoml.dumps, data)



from datetime import datetime, timedelta, timezone

import rustoml

obj = {
    "title": "TOML Example",
    "owner": {
        "dob": datetime(1979, 5, 27, 7, 32, tzinfo=timezone(timedelta(hours=-8))),
        "name": "Tom Preston-Werner",
    },
    "database": {
        "connection_max": 5000,
        "enabled": True,
        "ports": [8001, 8001, 8002],
        "server": "192.168.1.1",
    },
}

loaded_obj = rustoml.load("""\
# This is a TOML document.

title = "TOML Example"

[owner]
name = "Tom Preston-Werner"
dob = 1979-05-27T07:32:00-08:00 # First class dates

[database]
server = "192.168.1.1"
ports = [8001, 8001, 8002]
connection_max = 5000
enabled = true
""")

assert loaded_obj == obj

assert (
    rustoml.dumps(obj)
    == """\
title = "TOML Example"

[owner]
dob = 1979-05-27T07:32:00-08:00
name = "Tom Preston-Werner"

[database]
connection_max = 5000
enabled = true
server = "192.168.1.1"
ports = [8001, 8001, 8002]
"""
)



"""Nox sessions for testing across Python versions."""

import nox

nox.options.default_venv_backend = "uv"

PYTHON_VERSIONS: list[str] = ["3.10", "3.11", "3.12", "3.13", "3.14"]


@nox.session(python=PYTHON_VERSIONS)
def test(session: nox.Session) -> None:
    """Run tests for a specific Python version."""
    # Install maturin and build the package
    session.install("maturin")
    session.run("maturin", "develop", "--release")

    # Install test dependencies
    session.install("pytest", "dirty-equals")

    # Run tests
    session.run("pytest", "tests/", "-v")


@nox.session(python=PYTHON_VERSIONS)
def quick_test(session: nox.Session) -> None:
    """Quick smoke test: build and import."""
    session.install("maturin")
    session.run("maturin", "develop", "--release")

    # Quick import test
    session.run(
        "python",
        "-c",
        "import rustoml; import sys; print(f'✅ Python {sys.version.split()[0]} - rustoml v{rustoml.__version__}')",
    )


@nox.session
def test_all(session: nox.Session) -> None:
    """Run quick tests on all Python versions."""
    for version in PYTHON_VERSIONS:
        session.log(f"Testing Python {version}...")
        session.run("nox", "-s", f"quick_test-{version}", external=True)



from datetime import date, datetime, time, timedelta, timezone

import pytest

import rustoml


@pytest.mark.parametrize(
    "input_toml,output_obj",
    [
        ('"" = "bar"', {"": "bar"}),
        ('foo = "bar"', {"foo": "bar"}),
        ("ports = [ 8001, 8001, 8002 ]", {"ports": [8001, 8001, 8002]}),
        ("x = 1979-05-27T07:32:00", {"x": datetime(1979, 5, 27, 7, 32)}),
        ("x = 1979-05-27T07:32:00.123", {"x": datetime(1979, 5, 27, 7, 32, 0, 123000)}),
        ("x = 1979-05-27T07:32:00.123000", {"x": datetime(1979, 5, 27, 7, 32, 0, 123000)}),
        ("x = 1979-05-27T07:32:00.000123", {"x": datetime(1979, 5, 27, 7, 32, 0, 123)}),
        ("x = 1979-05-27T07:32:00Z", {"x": datetime(1979, 5, 27, 7, 32, tzinfo=timezone.utc)}),
        ("x = 1979-05-27T07:32:00-08:00", {"x": datetime(1979, 5, 27, 7, 32, tzinfo=timezone(timedelta(hours=-8)))}),
        ("x = 1979-05-27T07:32:00+08:00", {"x": datetime(1979, 5, 27, 7, 32, tzinfo=timezone(timedelta(hours=8)))}),
        (
            "x = 1979-05-27T07:32:00+08:15",
            {"x": datetime(1979, 5, 27, 7, 32, tzinfo=timezone(timedelta(hours=8, minutes=15)))},
        ),
        (
            "x = 1979-05-27T00:32:00.123-07:00",
            {"x": datetime(1979, 5, 27, 0, 32, 0, 123000, tzinfo=timezone(timedelta(hours=-7)))},
        ),
        ("x = 1979-05-27 07:32:00", {"x": datetime(1979, 5, 27, 7, 32)}),
        ("x = 1979-05-27T00:32:00.999999", {"x": datetime(1979, 5, 27, 0, 32, 0, 999999)}),
        ("x = 1987-01-28", {"x": date(1987, 1, 28)}),
        ("x = 12:34:56", {"x": time(12, 34, 56)}),
        ('foo.bar = "thing"', {"foo": {"bar": "thing"}}),
        (
            '''
foo.bar = """
thing"""
''',
            {"foo": {"bar": "thing"}},
        ),
        (
            '''
foo.bar = """
thing
"""
''',
            {"foo": {"bar": "thing\n"}},
        ),
        (
            '''
foo.bar = """
thing

more"""
''',
            {"foo": {"bar": "thing\n\nmore"}},
        ),
        (
            """
# This is a TOML document. (from https://github.com/toml-lang/toml)

title = "TOML Example"

[owner]
name = "Tom Preston-Werner"
dob = 1979-05-27T07:32:00-08:00 # First class dates

[database]
server = "192.168.1.1"
ports = [ 8001, 8001, 8002 ]
connection_max = 5000
enabled = true

[servers]

  # Indentation (tabs and/or spaces) is allowed but not required
  [servers.alpha]
  ip = "10.0.0.1"
  dc = "eqdc10"

  [servers.beta]
  ip = "10.0.0.2"
  dc = "eqdc10"

[clients]
data = [ ["gamma", "delta"], [1, 2] ]

# Line breaks are OK when inside arrays
hosts = [
  "alpha",
  "omega"
]
""",
            {
                "clients": {"data": [["gamma", "delta"], [1, 2]], "hosts": ["alpha", "omega"]},
                "database": {
                    "connection_max": 5000,
                    "enabled": True,
                    "ports": [8001, 8001, 8002],
                    "server": "192.168.1.1",
                },
                "owner": {
                    "dob": datetime(1979, 5, 27, 7, 32, tzinfo=timezone(timedelta(hours=-8))),
                    "name": "Tom Preston-Werner",
                },
                "servers": {"alpha": {"dc": "eqdc10", "ip": "10.0.0.1"}, "beta": {"dc": "eqdc10", "ip": "10.0.0.2"}},
                "title": "TOML Example",
            },
        ),
        (
            """
[[fruit]]
  name = "apple"

  [fruit.physical]  # subtable
    color = "red"
    shape = "round"

  [[fruit.variety]]  # nested array of tables
    name = "red delicious"

  [[fruit.variety]]
    name = "granny smith"

[[fruit]]
  name = "banana"

  [[fruit.variety]]
    name = "plantain"
""",
            {
                "fruit": [
                    {
                        "name": "apple",
                        "physical": {"color": "red", "shape": "round"},
                        "variety": [{"name": "red delicious"}, {"name": "granny smith"}],
                    },
                    {"name": "banana", "variety": [{"name": "plantain"}]},
                ]
            },
        ),
        ("""[ j . "ʞ" . 'l' ]""", {"j": {"ʞ": {"l": {}}}}),
    ],
)
def test_load(input_toml, output_obj):
    assert rustoml.load(input_toml) == output_obj


def test_load_str():
    assert rustoml.load('foo = "bar"') == {"foo": "bar"}


@pytest.mark.parametrize(
    "input_toml,output_obj",
    [
        ('foo = "bar"', {"foo": "bar"}),
        ('emoji = "😷"', {"emoji": "😷"}),
        ('polish = "Witaj świecie"', {"polish": "Witaj świecie"}),
    ],
)
def test_load_path(tmp_path, input_toml, output_obj):
    p = tmp_path / "test.toml"
    p.write_text(input_toml, encoding="UTF-8")
    assert rustoml.load(p) == output_obj


def test_load_file(tmp_path):
    p = tmp_path / "test.toml"
    p.write_text('foo = "bar"')
    with p.open() as f:
        assert rustoml.load(f) == {"foo": "bar"}


def test_invalid_type():
    with pytest.raises(TypeError, match="invalid toml input, must be str not <class 'bytes'>"):
        rustoml.load(b"foobar")


def test_datetime_tz_neg():
    d: datetime = rustoml.load("date = 1979-05-27T07:32:00.999999-08:00")["date"]
    assert d == datetime(1979, 5, 27, 7, 32, 0, 999999, tzinfo=timezone(timedelta(seconds=-28800)))
    assert d.tzinfo.tzname(datetime.now()) == "UTC-08:00"


def test_datetime_tz_pos():
    d: datetime = rustoml.load("date = 1979-05-27T07:32:00+05:15")["date"]
    assert d == datetime(1979, 5, 27, 7, 32, tzinfo=timezone(timedelta(seconds=18900)))
    assert d.tzinfo.tzname(datetime.now()) == "UTC+05:15"


def test_datetime_tz_utc():
    d: datetime = rustoml.load("date = 1979-05-27T07:32:00Z")["date"]
    assert d == datetime(1979, 5, 27, 7, 32, tzinfo=timezone.utc)
    assert d.tzinfo.tzname(datetime.now()) == "UTC"


def test_datetime_invalid():
    with pytest.raises(rustoml.TomlParsingError, match="day is out of range for month"):
        rustoml.load("date = 1979-02-30T07:32:00")


def test_invalid_toml():
    m = r"^invalid TOML value, did you mean to use a quoted string\? at line 1 column 5$"
    with pytest.raises(rustoml.TomlParsingError, match=m):
        rustoml.load("x = y")


def test_mixed_array():
    assert rustoml.loads("x = [1.1, 2, 3.3]") == {"x": [1.1, 2, 3.3]}
    assert rustoml.loads('x = [1, ["Arrays are not integers."]]') == {"x": [1, ["Arrays are not integers."]]}
    assert rustoml.loads('x = ["hi", 42]') == {"x": ["hi", 42]}


def test_subtable():
    """
    This is slightly incorrect, but matches normal TOML parsing.

    See https://github.com/alexcrichton/toml-rs/issues/367
    """
    s = """\
[fruit]
apple.color = "red"
apple.taste.sweet = true

[fruit.apple.texture]  # you can add sub-tables
smooth = true
"""
    with pytest.raises(rustoml.TomlParsingError, match="duplicate key: `apple` for key `fruit` at line 5 column 1"):
        rustoml.loads(s)


def test_none_value():
    assert rustoml.loads('x = "null"') == {"x": "null"}
    assert rustoml.loads('x = "null"', none_value="null") == {"x": None}
    assert rustoml.loads('x = "null"', none_value="") == {"x": "null"}
    assert rustoml.loads('x = ""', none_value="") == {"x": None}

    # Reproducible with the same none_value repr
    s = {"x": None}
    assert rustoml.dumps(s, none_value="py:None") == 'x = "py:None"\n'
    assert rustoml.loads('x = "py:None"\n', none_value="py:None") == s



#!/usr/bin/env python3
# used to test against https://github.com/BurntSushi/toml-test
# via `~/go/bin/toml-test ./tests/toml_test.py`
import json
import sys
from datetime import date, datetime, time
from pathlib import Path

ROOT_DIR = (Path(__file__).parent / "..").resolve()

sys.path.append(str(ROOT_DIR))

import rustoml  # noqa


def simplify(value):
    if isinstance(value, dict):
        return {k: simplify(v) for k, v in value.items()}
    elif isinstance(value, list):
        return [simplify(v) for v in value]
    elif isinstance(value, str):
        return {"type": "string", "value": value}
    elif isinstance(value, bool):
        return {"type": "bool", "value": str(value).lower()}
    elif isinstance(value, int):
        return {"type": "integer", "value": str(value)}
    elif isinstance(value, float):
        return {"type": "float", "value": repr(value)}
    elif isinstance(value, datetime):
        return {"type": "datetime", "value": value.isoformat().replace("+00:00", "Z")}
    elif isinstance(value, date):
        return {"type": "date", "value": value.isoformat()}
    elif isinstance(value, time):
        return {"type": "time", "value": value.strftime("%H:%M:%S.%f")}
    assert False, f"Unknown type: {type(value)}"


if __name__ == "__main__":
    data = rustoml.loads(sys.stdin.read())
    result = simplify(data)
    print(json.dumps(result, indent=2))



"""Type stubs for the rustoml Rust extension module."""

from typing import Any

__version__: str
"""The version of the rustoml package."""

def deserialize(toml: str, none_value: str | None = None) -> dict[str, Any]:
    """
    Deserialize a TOML string into a Python dictionary.

    Args:
        toml: A string containing valid TOML data.
        none_value: Optional string value to be converted to None. If specified,
            any string matching this value in the TOML will be deserialized as None.

    Returns:
        A dictionary representing the parsed TOML data.

    Raises:
        TomlParsingError: If the TOML string is invalid or cannot be parsed.
    """
    ...

def serialize(obj: Any, none_value: str | None = "null") -> str:
    """
    Serialize a Python object to a TOML string.

    Args:
        obj: A Python object to serialize (typically a dict).
        none_value: Optional string representation for None values. If None is passed,
            None values will be omitted from the output. Default is 'null'.

    Returns:
        A TOML string representation of the object.

    Raises:
        TomlSerializationError: If the object cannot be serialized to TOML.
    """
    ...

def serialize_pretty(obj: Any, none_value: str | None = "null") -> str:
    """
    Serialize a Python object to a pretty-formatted TOML string.

    Args:
        obj: A Python object to serialize (typically a dict).
        none_value: Optional string representation for None values. If None is passed,
            None values will be omitted from the output. Default is 'null'.

    Returns:
        A pretty-formatted TOML string representation of the object.

    Raises:
        TomlSerializationError: If the object cannot be serialized to TOML.
    """
    ...

class TomlParsingError(ValueError):
    """Exception raised when TOML parsing fails."""

    ...

class TomlSerializationError(ValueError):
    """Exception raised when TOML serialization fails."""

    ...
